;; grouplock.jl - Make windows of the same group be locked together


;; Original-Author: Yann Hodique <Yann.Hodique@lifl.fr>
;; Original-Author: Scott Scriven <sawfish@toykeeper.net>
;; Author: Christopher Bratusek <nano-master@gmx.de>
;; Author: Fernando Carmona Varo <ferkiwi@gmail.com>

(provide 'grouplock)


(defgroup groups "Groups")


(defcustom group-lock t
  "Stick together all the windows of the same group. 
The whole group will be affected when moving, resizing, (maxi/mini)mizing, [un]shading and stacking."
  :group groups
  :type boolean)



(defvar group-refresh-lock t
  "Lock used by `refresh-group-lock' to avoid concurrent refreshing.")




(defun move-resize-frame-window-to (win x y w h)
  "Move and resize according to *frame* dimensions"
  (let* ((dim1 (window-dimensions win))
         (dim2 (window-frame-dimensions win))
         (dw (- (car dim2) (car dim1)))
         (dh (- (cdr dim2) (cdr dim1))))
    (move-resize-window-to win x y (- w dw) (- h dh)))
  )





(defun refresh-group-lock (win prop)
  "Refresh the entire group containing win according to prop
prop can be one of the symbols : frame, move, resize, shade, unshade, add"
  (if group-lock
      (progn
        (setq group-refresh-lock nil)
        (case prop
          ((frame) ; rebuild all frames
           (map-window-group rebuild-frame win)
           )
          ((move resize) ; Move-resize whole group
           (let ((dim (window-frame-dimensions win))
                 (pos (window-position win)))
             (map-window-group (lambda (w) 
                                 (move-resize-frame-window-to w (car pos) (cdr pos) (car dim) (cdr dim))
                                 (rebuild-frame w)) win)
             ))
           ((stick) ; Make them all sticky
            (map-window-group (lambda (w)
                                 (toggle-window-sticky w)) win)
            )
           ((shade) ; Make all shaded
            (map-window-group (lambda (w)
                                (shade-window w)
                                (rebuild-frame w)) wins)
            )
           ((unshade) ; Unshade them all
            (map-window-group (lambda (w)
                                (unshade-window w)
                                (rebuild-frame w)) wins)
            )
           ((add) ; Change new window to the position and size of the rest of the members
            (let ((member (car (windows-in-group win))))
              (when (not (eq member win))
                (let ((dim (window-frame-dimensions member))
                      (pos (window-position member)))
                  (map-window-group 
                   (lambda (w) 
                     (move-resize-frame-window-to w (car pos) (cdr pos) (car dim) (cdr dim))
                     (rebuild-frame w)) win))))
            )
           ((sort) ; Rearrange windows stacking
            (map-window-group raise-window win)
            (raise-window win)
            )
           ))
    (setq group-refresh-lock t)
    )
  )




(add-hook 'after-add-window-hook (lambda (win) (refresh-group-lock win 'add)))
;;(add-hook 'add-window-to-group-hook (lambda (win) (refresh-group-lock win 'add))))

(add-hook 'after-move-hook (lambda (win args) (refresh-group-lock win 'move)))
(add-hook 'while-moving-hook (lambda (win) (when (eq move-resize-mode 'opaque) 
                                             (refresh-group-lock win 'move))))

(add-hook 'after-resize-hook (lambda (win args) (refresh-group-lock win 'resize)))
(add-hook 'while-resizing-hook (lambda (win) (when (eq move-resize-mode 'opaque) 
                                               (refresh-group-lock win 'resize))))

(add-hook 'window-maximized-hook (lambda (win args) (refresh-group-lock win 'resize)))
(add-hook 'window-unmaximized-hook (lambda (win args) (refresh-group-lock win 'resize)))

(add-hook 'shade-window-hook (lambda (win) (refresh-group-lock win 'shade)))
(add-hook 'unshade-window-hook (lambda (win) (refresh-group-lock win 'unshade)))

(add-hook 'focus-in-hook (lambda (win) (refresh-group-lock win 'sort)))



