;; tab.jl - Emulate fluxbox tabs system

;; version 1.0.0

;; Original-Author: Yann Hodique <Yann.Hodique@lifl.fr>
;; Original-Author: Scott Scriven <sawfish@toykeeper.net>
;; Author: Christopher Bratusek <nano-master@gmx.de>
;; Author: Fernando Carmona varo <ferkiwi@gmail.com>


(provide 'tab)

(require 'marks)
(require 'grouplock)


(defcustom tab-left-dec-width 11 "Width of tab's left-edge decoration"
  :group groups
  :type number)

(defcustom tab-right-dec-width 11 "Width of tab's right-edge decoration"
  :group groups
  :type number)

(defcustom tab-left-margin 16 "Width of tab area's left-edge decoration"
  :group groups
  :type number)

(defcustom tab-right-margin 16 "Width of tab area's right-edge decoration"
  :group groups
  :type number)

(defcustom tab-raise-on-hover t "Raise Tabs on Hover"
  :group groups
  :type boolean)


(defun rank-in-list (elem list)
  "Returns the position of the given elem in the list.
If elem is not found, returns nil."
  (cond
   ((not list) 0)
   ((eq elem (car list)) 0)
   (t (+ 1 (rank-in-list elem (cdr list))))))

(defun get-tab-pos (win)
  (let* ((group  (window-actual-group-id win))
         (tabnum (rank-in-list win (windows-by-group group)))
         )
    (tab-pos group tabnum win)
    )
  )

(defun tab-pos (group tabnum win)
  "find the left and right pixel offsets of a tab"
  (let* (
         (tabarea-width (+
                         ;; get width of a window in this group
                         ;;(car (window-dimensions (car (tab-group-window-list group))))
                         (car (window-dimensions win))
                         (- tab-left-margin)
                         (- tab-right-margin)
                         )
                        )
         (numtabs (length (windows-by-group group)))
         (left (quotient (* tabnum tabarea-width) numtabs))
         ;; the right edge is not always "left + (window-width / numtabs)"
         ;; that would be inaccurate due to rounding errors
         (right (quotient (* (+ tabnum 1) tabarea-width) numtabs))
         (width (- right left))
         )
    (list left right width)
    )
  )

(defun tab-title-text-width (win)
  "width of the title text area is the tabwidth minus decorations"
  (let* ((tabwidth (nth 2 (get-tab-pos win))))
    (+ tabwidth
       (- tab-left-dec-width)
       (- tab-right-dec-width)
       )
    )
  )

(defun tab-left-edge (win)
  "Compute left edge of tab"
  (let* ((left (nth 0 (get-tab-pos win))))
    (+ left tab-left-margin)
    )
  )

(defun tab-right-dec-pos (win)
  "Compute position of tab's right-edge decoration"
  (let* ((right (nth 1 (get-tab-pos win))))
    (+ right tab-left-margin (- tab-right-dec-width))
    )
  )

(defun tab-title-left-edge (win)
  "Compute left edge of tab"
  (+ (tab-left-edge win) tab-left-dec-width))

;; new class : tab
(def-frame-class tab
  `((cursor . left_ptr)
    (x-justify . center)
    (y-justify . center)
    ;;(left-edge . ,tab-window-group-left-edge)
    (left-edge . ,tab-title-left-edge)
    (width . ,tab-title-text-width)
    )
  ;;(bind-keys tab-keymap "Button1-Move" 'move-window-interactively)
  (bind-keys tab-keymap)
  )

(def-frame-class tab-l
  `((cursor . left_ptr)
    (left-edge . ,tab-left-edge)
    )
  (bind-keys tab-keymap)
  )

(def-frame-class tab-r
  `((cursor . left_ptr)
    (left-edge . ,tab-right-dec-pos)
    ;;(right-edge . 0)
    )
  (bind-keys tab-keymap)
  )



(defun tab-group-window (w win)
  "Put active window in pointer-selected group"
  (interactive)
  (let ((old (window-actual-group-id w))
        (new (window-actual-group-id win))
        (glist (window-group-ids)))
    (add-window-to-group w new)
    (refresh-group-lock win 'move) ; refresh new group and move in position
    (unless (eq (rank-in-list old glist) (length glist) ) ; refresh old group if exists 
      ;; ;; Something is wrong with the line below.. I'm not sure why
      ;;(refresh-group-lock (car (windows-by-group old)) 'frame)
      )
    ))


(defun mygroup (win)
  (interactive "%W")
  (if (marked-windows)
      (progn
        (apply-on-marked-windows (lambda (w) (tab-group-window w win)))
        (unmark-all-windows))
    (mark-window win)))


(if (eq tab-raise-on-hover 't)
    (add-hook 'enter-frame-part-hook
              (lambda (win pclass fmode)
                (raise-window win)
                )))

;; Refresh window frame after closing a tab
(add-hook 'destroy-notify-hook (lambda (win) (refresh-group-lock win 'frame)))

